from random import choice

from constance.tld import domains, tlds

from ..personal.names import fake_name, fake_surname
from ..strings import fake_string


def fake_email(addr=None, domain=None, tld=None):
    addr = addr or f"{fake_string(min_length=1)}"
    domain = domain or f"{fake_string(min_length=1)}"
    tld = tld or f"{fake_string(min_length=2, max_length=7)}"
    return f"{addr}@{domain}.{tld}"


def fake_enough_email(addr=None, domain=None, tld=None):
    addr = addr or f"{fake_name()}.{fake_surname()}"
    domain = domain or f"{choice(domains)}"
    tld = tld or f"{choice(tlds)}"
    return f"{addr}@{domain}.{tld}"



from uuid import uuid1, uuid3, uuid4, uuid5


def fake_uuid1(node=None, clock_seq=None):
    return uuid1(node, clock_seq)


def fake_uuid3(namespace, name):
    return uuid3(namespace, name)


def fake_uuid4():
    return uuid4()


def fake_uuid5(namespace, name):
    return uuid5(namespace, name)


def fake_uuid():
    return fake_uuid4()



from random import choice

names = [
    "James",
    "John",
    "Robert",
    "Michael",
    "William",
    "David",
    "Richard",
    "Joseph",
    "Thomas",
    "Charles",
    "Christopher",
    "Daniel",
    "Matthew",
    "Anthony",
    "Donald",
    "Mark",
    "Paul",
    "Steven",
    "Andrew",
    "Kenneth",
    "George",
    "Joshua",
    "Kevin",
    "Brian",
    "Edward",
    "Ronald",
    "Timothy",
    "Jason",
    "Jeffrey",
    "Ryan",
    "Jacob",
    "Gary",
    "Nicholas",
    "Eric",
    "Stephen",
    "Jonathan",
    "Larry",
    "Justin",
    "Scott",
    "Brandon",
    "Frank",
    "Benjamin",
    "Gregory",
    "Raymond",
    "Samuel",
    "Patrick",
    "Alexander",
    "Jack",
    "Dennis",
    "Jerry",
    "Tyler",
    "Aaron",
    "Henry",
    "Jose",
    "Douglas",
    "Peter",
    "Adam",
    "Nathan",
    "Zachary",
    "Walter",
    "Kyle",
    "Harold",
    "Carl",
    "Jeremy",
    "Gerald",
    "Keith",
    "Roger",
    "Arthur",
    "Terry",
    "Lawrence",
    "Sean",
    "Christian",
    "Ethan",
    "Austin",
    "Joe",
    "Albert",
    "Jesse",
    "Willie",
    "Billy",
    "Bryan",
    "Bruce",
    "Noah",
    "Jordan",
    "Dylan",
    "Ralph",
    "Roy",
    "Alan",
    "Wayne",
    "Eugene",
    "Juan",
    "Gabriel",
    "Louis",
    "Russell",
    "Randy",
    "Vincent",
    "Philip",
    "Logan",
    "Bobby",
    "Harry",
    "Johnny",
]

surnames = [
    "Smith",
    "Jones",
    "Taylor",
    "Williams",
    "Brown",
    "Davies",
    "Evans",
    "Wilson",
    "Thomas",
    "Roberts",
    "Johnson",
    "Lewis",
    "Walker",
    "Robinson",
    "Wood",
    "Thompson",
    "White",
    "Watson",
    "Jackson",
    "Wright",
    "Green",
    "Harris",
    "Cooper",
    "King",
    "Lee",
    "Martin",
    "Clarke",
    "James",
    "Morgan",
    "Hughes",
    "Edwards",
    "Hill",
    "Moore",
    "Clark",
    "Harrison",
    "Scott",
    "Young",
    "Morris",
    "Hall",
    "Ward",
    "Turner",
    "Carter",
    "Phillips",
    "Mitchell",
    "Patel",
    "Adams",
    "Campbell",
    "Anderson",
    "Allen",
    "Cook",
    "Bailey",
    "Parker",
    "Miller",
    "Davis",
    "Murphy",
    "Price",
    "Bell",
    "Baker",
    "Griffiths",
    "Kelly",
    "Simpson",
    "Marshall",
    "Collins",
    "Bennett",
    "Cox",
    "Richardson",
    "Fox",
    "Gray",
    "Rose",
    "Chapman",
    "Hunt",
    "Robertson",
    "Shaw",
    "Reynolds",
    "Lloyd",
    "Ellis",
    "Richards",
    "Russell",
    "Wilkinson",
    "Khan",
    "Graham",
    "Stewart",
    "Reid",
    "Murray",
    "Powell",
    "Palmer",
    "Holmes",
    "Rogers",
    "Stevens",
    "Walsh",
    "Hunter",
    "Thomson",
    "Matthews",
    "Ross",
    "Owen",
    "Mason",
    "Knight",
    "Kennedy",
    "Butler",
    "Saunders",
    "Cole",
    "Pearce",
    "Dean",
    "Foster",
    "Harvey",
    "Hudson",
    "Gibson",
    "Mills",
    "Berry",
    "Barnes",
    "Pearson",
    "Kaur",
    "Booth",
    "Dixon",
    "Grant",
    "Gordon",
    "Lane",
    "Harper",
    "Ali",
    "Hart",
    "Mcdonald",
    "Brooks",
    "Ryan",
    "Carr",
    "Macdonald",
    "Hamilton",
    "Johnston",
    "West",
    "Gill",
    "Dawson",
    "Armstrong",
    "Gardner",
    "Stone",
    "Andrews",
    "Williamson",
    "Barker",
    "George",
    "Fisher",
    "Cunningham",
    "Watts",
    "Webb",
    "Lawrence",
    "Bradley",
    "Jenkins",
    "Wells",
    "Chambers",
    "Spencer",
    "Poole",
    "Atkinson",
    "Lawson",
]


def fake_name():
    return choice(names)


def fake_surname():
    return choice(surnames)


def fake_fullname():
    return choice(names) + " " + choice(surnames)



from ipaddress import IPV4LENGTH, IPv4Address
from random import choice

from basics.numerics import fake_int


def fake_ipv4() -> IPv4Address:
    return IPv4Address(fake_int(0, 2 ** IPV4LENGTH))


def fake_ipv4_24() -> IPv4Address:
    return IPv4Address(fake_int(10 * 2 ** 24, 11 * 2 ** 24))


def fake_ipv4_20() -> IPv4Address:
    return IPv4Address(fake_int(2886729728, 2887778304))


def fake_ipv4_16() -> IPv4Address:
    return IPv4Address(fake_int(3232235520, 3232301056))


def fake_ipv4_local() -> IPv4Address:
    return choice((fake_ipv4_16(), fake_ipv4_20(), fake_ipv4_24()))


def fake_ipv4_broadcast_netwok() -> IPv4Address:
    return IPv4Address(2 ** 32 - 1)


def fake_ipv4_broadcast() -> IPv4Address:
    return IPv4Address(fake_int(3221225984, 3221226240))


def fake_ipv4_multicast() -> IPv4Address:
    return IPv4Address(fake_int(3758096384, 4026531840))



from base64 import b16encode, b32encode, b64encode
from os import urandom
from random import randint

from fakeit.exceptions import InvalidRange


def fake_bytes(min_length=0, max_length=16):
    if max_length < min_length:
        raise InvalidRange

    length = randint(min_length, max_length)
    return urandom(length)


def fake_b64(min_length=0, max_length=16):
    return b64encode(fake_bytes(min_length, max_length))


def fake_b32(min_length=0, max_length=16):
    return b32encode(fake_bytes(min_length, max_length))


def fake_b16(min_length=0, max_length=16):
    return b16encode(fake_bytes(min_length, max_length))



class BaseFakeItException(Exception):
    message = None

    def __init__(self, message=None):
        if not message:
            message = self.message
        super().__init__(message)


class FakeItException(BaseFakeItException):
    message = "Unknown exception"


class OutOfRangeException(BaseFakeItException):
    message = "Out of range"


class EmptyException(BaseFakeItException):
    message = "Empty value"


class InvalidRange(BaseFakeItException):
    message = "Ivalid range"


class EmptyAlphabet(BaseFakeItException):
    message = "Empty alphabet"


class InvalidValue(BaseFakeItException):
    message = "Invalid value"


class AdditionalRequirementError(BaseFakeItException):
    message = "Install additional requirement for use submodule. ({outer_error})"

    def __init__(self, message=None):
        if not message:
            message = self.message
        message = self.message.format(outer_error=message)
        super().__init__(message)


class FuckIt(BaseFakeItException):
    message = "F*CK"



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['fakeit',
 'fakeit.advanced',
 'fakeit.advanced.person',
 'fakeit.basics',
 'fakeit.basics.boolean',
 'fakeit.basics.bytes',
 'fakeit.basics.geo',
 'fakeit.basics.hashes',
 'fakeit.basics.ip',
 'fakeit.basics.numerics',
 'fakeit.basics.personal',
 'fakeit.basics.strings',
 'fakeit.basics.texts',
 'fakeit.basics.uuids',
 'fakeit.constance',
 'fakeit.constance.email',
 'fakeit.constance.tld',
 'fakeit.extras',
 'fakeit.extras.alchemy',
 'fakeit.extras.django']

package_data = \
{'': ['*']}

setup_kwargs = {
    'name': 'fakeit',
    'version': '0.1.8',
    'description': 'Generate fake data',
    'long_description': '# Meet the FAKE IT project which provide generating simple fake data\n\n-----\n\n## INSTALL:\n- python setup.py install\n- pip install fakeit\n- pip install git+https://github.com/RCheese/fakeit\n\n## USAGE:\nGenerating fake data for python types\n\n### Bytes\n```python\n    >>> from fakeit import bytes\n    >>> bytes.fake_bytes(min_length=2, max_length=5)\n    ... b\'/\\xf5Q\\x9a\\xcd\'\n    \n    >>> bytes.fake_b64(min_length=2, max_length=5)\n    ... b\'QpwJug==\'\n```\n\n### Hashes\n```python\n    >>> from fakeit import hashes\n    >>> hashes.fake_md5()\n    ... \'37c6c63ee4dd8516d3d8ee4319b3e7b8\'\n    >>> hashes.fake_sha256()\n    ... \'bc6c64d150e869cf10e3d9c0cf582fa78fe46282de75911b464c46a023a08038\'\n```\n\n### Numerics\n```python\n    >>> from fakeit import numerics\n    >>> numerics.fake_complex(1,2,3,4)\n    ... (1.2157335093960198+3.17909803327301j)\n    >>> numerics.fake_complex(1,2,3,4, round=True)\n    ... (2+4j)\n    >>> numerics.fake_int(1 ,20)\n    ... 11\n    >>> numerics.fake_float(1, 20)\n    ... 3.448023122876366\n```\n\n### Strings\n```python\n    >>> from fakeit import strings\n    >>> strings.fake_string(min_length=5, max_length=5)\n    ... \'CPOcO\'\n    \n    >>> fake_string(min_length=5, max_length=5, unique=True)\n    ... \'qEiwW\'\n    \n    >>> fake_string(min_length=5, max_length=5, alphabet="ABCdE")\n    ... \'EdBdB\'\n    \n    >>> fake_strings(5)\n    ... <generator object fake_strings at 0x7f67579e4660>\n    \n    >>> for i in fake_strings(5):\n    >>>     print(i)\n    >>>\n    ... 1r3OxTKis20KF\n    ... \n    ... 3YN28kOPLuc\n    ... DaLQ\n    ... j7J9MMJcF2\n    \n    >>> for i in all_combinations_with_replacement_fake_string(min_length=3, max_length=3, alphabet="abc"):\n    >>>     print(i)\n    ... aaa\n    ... aab\n    ... aac\n    ... abb\n    ... abc\n    ... acc\n    ... bbb\n    ... bbc\n    ... bcc\n    ... ccc\n```\n\n### Personal\n``\n```python\n    >>> from fakeit import personal\n    >>> personal.names.fake_fullname()\n    ... \'Justin Hall\'\n    >>> personal.names.fake_name()\n    ... \'Johnny\'\n    >>> personal.names.fake_surname()\n    ... \'Gill\'\n    \n    >>> personal.phones.fake_international()\n    ... \'+67-910-8211582\'\n    >>> personal.phones.fake_international(mediator="")\n    ... \'+974665503991\'\n    >>> fake_international(country_code=7, area_code=923)\n    ... \'+7-923-4915850\'\n    \n    >>> personal.emails.fake_email()\n    ... \'0mfJz0QD@VqujvpRDiuMfuyB.VRQfn\'\n    >>> personal.emails.fake_enough_email()\n    ... \'Henry.Hill@google.cn\'\n    \n    >>> personal.fake_person()\n    ... <Person Stephen Robertson> (FirstName=Stephen, LastName=Robertson, Email=Stephen.Robertson@whatsapp.net, Phone=+1-990-5674435)\n```\n\n## TODO:\n\n- Geo\n    - Position\n    - Named (Country, City, and etc.)\n    - Address\n- Text data\n- Tables\n    - ?\n- Unit tests\n- SQLAlchemy type casting\n- Django type casting\n- Sphinx docs\n- CI\n- Compilation request\n\n<a href="https://www.buymeacoffee.com/RussianCheese" target="_blank"><img src="https://cdn.buymeacoffee.com/buttons/arial-violet.png" alt="Buy Me A Coffee" style="height: 51px !important;width: 217px !important;" ></a>\n',
    'author': 'Ruslan Samoylov',
    'author_email': 'ruslan.v.samoylov@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'python_requires': '>=3.6',
}


setup(**setup_kwargs)



from copy import deepcopy
from functools import wraps

from fakeit.basics.boolean import fake_bool
from fakeit.basics.numerics import fake_float, fake_int
from fakeit.basics.strings import fake_string
from sqlalchemy.ext.declarative import DeclarativeMeta
from sqlalchemy.sql.schema import Table


def _tableconvert(func):
    @wraps(func)
    def wrapped(cls_or_rbl, *args, **kwargs):
        if isinstance(cls_or_rbl, Table):
            table = cls_or_rbl
        elif isinstance(cls_or_rbl, DeclarativeMeta):
            table = cls_or_rbl.__table__
        else:
            raise TypeError
        return func(table, *args, **kwargs)

    return wrapped


@_tableconvert
def get_not_nullable(table):
    return (x for x in table.c if not x.nullable)


def python_type_cast(column):
    return column.type.python_type


@_tableconvert
def get_template(table):
    return dict((column.name, column.type) for column in table.c)


@_tableconvert
def get_basic_template(table):
    return dict((column.name, column.type) for column in table.c if not column.nullable)


type_mapping = {str: fake_string, int: fake_int, float: fake_float, bool: fake_bool}


def fill_template(template, id=None, **kwargs):
    tmpl = deepcopy(template)
    res = {}
    if "id" in tmpl:
        tmpl.pop("id")
        if id is None:
            id = fake_int(0, 128)

    for k, v in tmpl.items():
        if k in kwargs:
            res[k] = kwargs[k]() if callable(kwargs[k]) else kwargs[k]
        elif v.python_type == int:
            res[k] = type_mapping[int](0, 777)
        elif v.python_type == float:
            res[k] = type_mapping[float](0, 777)
        elif v.python_type == str:
            res[k] = type_mapping[str](max_length=v.length)
        elif v.python_type == bool:
            res[k] = type_mapping[bool]()
        else:
            res[k] = None

    if ("id" in template) or id:
        res["id"] = id
    return res



tlds = {
    "com",
    "co.uk",
    "org",
    "net",
    "ru",
    "eu",
    "de",
    "fr",
    "tv",
    "cn",
    "es",
    "com.tr",
}
domains = {
    "google",
    "youtube",
    "facebook",
    "baidu",
    "wikipedia",
    "qq",
    "tmall",
    "yahoo",
    "taobao",
    "amazon",
    "sohu",
    "twitter",
    "jd",
    "live",
    "vk",
    "instagram",
    "weibo",
    "sina",
    "login",
    "360",
    "reddit",
    "linkedin",
    "blogspot",
    "netflix",
    "microsoftonline",
    "pages",
    "yandex",
    "mail",
    "pornhub",
    "twitch",
    "porn555",
    "csdn",
    "alipay",
    "microsoft",
    "aliexpress",
    "t",
    "ebay",
    "bing",
    "naver",
    "github",
    "stackoverflow",
    "livejasmin",
    "office",
    "msn",
    "imdb",
    "xvideos",
    "whatsapp",
    "tribunnews",
    "wordpress",
    "googleusercontent",
    "paypal",
    "bilibili",
    "apple",
    "xhamster",
    "imgur",
    "xinhuanet",
    "adobe",
    "fandom",
    "babytree",
    "pinterest",
    "fbcdn",
    "dropbox",
    "tumblr",
    "exosrv",
    "hao123",
    "amazonaws",
    "instructure",
    "speakol",
    "booking",
    "zhihu",
    "tianya",
    "soso",
    "thestartmagazine",
    "detail",
    "bbc",
    "salesforce",
    "pixnet",
    "aparat",
    "cnn",
    "onlinesbi",
    "force",
    "rakuten",
    "bodelen",
    "1cnblogs",
}



lorem_text = """
Lorem ipsum dolor sit amet, consectetur adipiscing elit, 
sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. 
Ut enim ad minim veniam, quis nostrud exercitation ullamco laboris 
nisi ut aliquip ex ea commodo consequat. Duis aute irure dolor in 
reprehenderit in voluptate velit esse cillum dolore eu fugiat nulla 
pariatur. Excepteur sint occaecat cupidatat non proident, sunt in 
culpa qui officia deserunt mollit anim id est laborum.

Curabitur pretium tincidunt lacus. Nulla gravida orci a odio. 
Nullam varius, turpis et commodo pharetra, est eros bibendum elit, 
nec luctus magna felis sollicitudin mauris. Integer in mauris eu 
nibh euismod gravida. Duis ac tellus et risus vulputate vehicula. 
Donec lobortis risus a elit. Etiam tempor. Ut ullamcorper, ligula eu 
tempor congue, eros est euismod turpis, id tincidunt sapien risus a quam. 
Maecenas fermentum consequat mi. Donec fermentum. 
Pellentesque malesuada nulla a mi. Duis sapien sem, aliquet nec, 
commodo eget, consequat quis, neque. Aliquam faucibus, 
elit ut dictum aliquet, felis nisl adipiscing sapien, 
sed malesuada diam lacus eget erat. Cras mollis scelerisque nunc. 
Nullam arcu. Aliquam consequat. Curabitur augue lorem, dapibus quis, 
laoreet et, pretium ac, nisi. Aenean magna nisl, mollis quis, molestie eu, 
feugiat in, orci. In hac habitasse platea dictumst.
"""


def lorem_ipsum():
    while True:
        yield lorem_text
